--  2001 Peter Thiemann
module HTMLSuperBase (module HTMLBase, module HTMLSuperBase) where

-- typed layer
-- attributes

data ATTRIBUTE a => ATTR a =				    -- a phantom type
  ATTR { unATTR :: ATTR_ }


class Show a => ATTRIBUTE a where
  show_name :: a -> String
-- 
  show_name = map toLower . show

-- attribute values

class (ATTRIBUTE a, Show v) => AttrValue a v

mkAttr :: (AttrValue a v) => a -> v -> ATTR a
mkAttr a v = ATTR (attr_ (show_name a) (show v))

attr a v into = add_attr into (mkAttr a v)

--  elements

data TAG t => ELT t =
    ELT { unELT :: ELEMENT_ }

class Show t => TAG t where
  make :: t -> ELT t
  show_tag :: t -> String
--
  make = make_standard
  show_tag = map toLower . show

make_standard t = ELT (element_ (show_tag t) [] [])
make_empty    t = ELT (empty_   (show_tag t) [])

--
class NextState s t s' | s t -> s'
class FinalState i f | f -> i

add :: (NextState s t s') => ELT s -> ELT t -> ELT s'
add (ELT e_) (ELT e'_) =
  ELT (add_ e_ e'_)

add' :: (NextState s t s') => ELT t -> ELT s -> ELT s'
add' = flip add

class AddAttr t a

add_attr :: (AddAttr t a) => ELT t -> ATTR a -> ELT t
add_attr (ELT e_) (ATTR att) =
  ELT (add_attr_ e_ att)

attr' :: (AddAttr t a) => ATTR a -> ELT t -> ELT t
attr' = flip add_attr
 
-- --================= FUNCTIONS =================----

-- --show functions------------------

show_html (ELT e_) = show e_

-- creating text elements

text :: (NextState s CDATA s') => String -> ELT s -> ELT s'
text str = add' (ELT (cdata_ str) :: ELT CDATA)

data CDATA = CDATA deriving Show
instance TAG CDATA
instance FinalState CDATA CDATA

-- creating the main document

data DOCUMENT = DOCUMENT deriving Show
instance TAG DOCUMENT where
  make DOCUMENT = 
    ELT (doctype_
  	  ["HTML"
          ,"PUBLIC"
	  ,"\"-//W3C//DTD HTML 4.01//EN\""
	  ,"\"http://www.w3.org/TR/html4/strict.dtd\""]
	  [])

-- instance AddTo DOCUMENT HTML -- must be in HTMLPrelude

show_document :: ELT DOCUMENT -> String
show_document = show_html
